@@ -5,56 +5,116 @@ set -euo pipefail
 clean_deep_system() {
     stop_section_spinner
     local cache_cleaned=0
-    safe_sudo_find_delete "/Library/Caches" "*.cache" "$MOLE_TEMP_FILE_AGE_DAYS" "f" && cache_cleaned=1 || true
-    safe_sudo_find_delete "/Library/Caches" "*.tmp" "$MOLE_TEMP_FILE_AGE_DAYS" "f" && cache_cleaned=1 || true
-    safe_sudo_find_delete "/Library/Caches" "*.log" "$MOLE_LOG_AGE_DAYS" "f" && cache_cleaned=1 || true
+    # Optimized: Single pass for /Library/Caches (3 patterns in 1 scan)
+    if sudo test -d "/Library/Caches" 2> /dev/null; then
+        while IFS= read -r -d '' file; do
+            if should_protect_path "$file"; then
+                continue
+            fi
+            if safe_sudo_remove "$file"; then
+                cache_cleaned=1
+            fi
+        done < <(sudo find "/Library/Caches" -maxdepth 5 -type f \( \
+            \( -name "*.cache" -mtime "+$MOLE_TEMP_FILE_AGE_DAYS" \) -o \
+            \( -name "*.tmp" -mtime "+$MOLE_TEMP_FILE_AGE_DAYS" \) -o \
+            \( -name "*.log" -mtime "+$MOLE_LOG_AGE_DAYS" \) \
+            \) -print0 2> /dev/null || true)
+    fi
     [[ $cache_cleaned -eq 1 ]] && log_success "System caches"
+    start_section_spinner "Cleaning system temporary files..."
     local tmp_cleaned=0
     safe_sudo_find_delete "/private/tmp" "*" "${MOLE_TEMP_FILE_AGE_DAYS}" "f" && tmp_cleaned=1 || true
     safe_sudo_find_delete "/private/var/tmp" "*" "${MOLE_TEMP_FILE_AGE_DAYS}" "f" && tmp_cleaned=1 || true
+    stop_section_spinner
     [[ $tmp_cleaned -eq 1 ]] && log_success "System temp files"
+    start_section_spinner "Cleaning system crash reports..."
     safe_sudo_find_delete "/Library/Logs/DiagnosticReports" "*" "$MOLE_CRASH_REPORT_AGE_DAYS" "f" || true
+    stop_section_spinner
     log_success "System crash reports"
-    safe_sudo_find_delete "/private/var/log" "*.log" "$MOLE_LOG_AGE_DAYS" "f" || true
-    safe_sudo_find_delete "/private/var/log" "*.gz" "$MOLE_LOG_AGE_DAYS" "f" || true
-    log_success "System logs"
-    if [[ -d "/Library/Updates" && ! -L "/Library/Updates" ]]; then
-        if ! is_sip_enabled; then
-            local updates_cleaned=0
-            while IFS= read -r -d '' item; do
-                if [[ -z "$item" ]] || [[ ! "$item" =~ ^/Library/Updates/[^/]+$ ]]; then
-                    debug_log "Skipping malformed path: $item"
-                    continue
-                fi
-                local item_flags
-                item_flags=$($STAT_BSD -f%Sf "$item" 2> /dev/null || echo "")
-                if [[ "$item_flags" == *"restricted"* ]]; then
-                    continue
-                fi
-                if safe_sudo_remove "$item"; then
-                    ((updates_cleaned++))
-                fi
-            done < <(find /Library/Updates -mindepth 1 -maxdepth 1 -print0 2> /dev/null || true)
-            [[ $updates_cleaned -gt 0 ]] && log_success "System library updates"
-        fi
+    start_section_spinner "Cleaning system logs..."
+    # Optimized: Single pass for /private/var/log (2 patterns in 1 scan)
+    if sudo test -d "/private/var/log" 2> /dev/null; then
+        while IFS= read -r -d '' file; do
+            if should_protect_path "$file"; then
+                continue
+            fi
+            safe_sudo_remove "$file" || true
+        done < <(sudo find "/private/var/log" -maxdepth 5 -type f \( \
+            -name "*.log" -o -name "*.gz" \
+            \) -mtime "+$MOLE_LOG_AGE_DAYS" -print0 2> /dev/null || true)
     fi
+    stop_section_spinner
+    log_success "System logs"
+    start_section_spinner "Scanning system library updates..."
+    if [[ -d "/Library/Updates" && ! -L "/Library/Updates" ]]; then
+        local updates_cleaned=0
+        while IFS= read -r -d '' item; do
+            if [[ -z "$item" ]] || [[ ! "$item" =~ ^/Library/Updates/[^/]+$ ]]; then
+                debug_log "Skipping malformed path: $item"
+                continue
+            fi
+            local item_flags
+            item_flags=$($STAT_BSD -f%Sf "$item" 2> /dev/null || echo "")
+            if [[ "$item_flags" == *"restricted"* ]]; then
+                continue
+            fi
+            if safe_sudo_remove "$item"; then
+                ((updates_cleaned++))
+            fi
+        done < <(find /Library/Updates -mindepth 1 -maxdepth 1 -print0 2> /dev/null || true)
+        stop_section_spinner
+        [[ $updates_cleaned -gt 0 ]] && log_success "System library updates"
+    else
+        stop_section_spinner
+    fi
+    start_section_spinner "Scanning macOS installer files..."
     if [[ -d "/macOS Install Data" ]]; then
         local mtime=$(get_file_mtime "/macOS Install Data")
         local age_days=$((($(get_epoch_seconds) - mtime) / 86400))
-        debug_log "Found macOS Install Data (age: ${age_days} days)"
-        if [[ $age_days -ge 30 ]]; then
+        debug_log "Found macOS Install Data, age ${age_days} days"
+        if [[ $age_days -ge 14 ]]; then
             local size_kb=$(get_path_size_kb "/macOS Install Data")
             if [[ -n "$size_kb" && "$size_kb" -gt 0 ]]; then
                 local size_human=$(bytes_to_human "$((size_kb * 1024))")
-                debug_log "Cleaning macOS Install Data: $size_human (${age_days} days old)"
+                debug_log "Cleaning macOS Install Data: $size_human, ${age_days} days old"
                 if safe_sudo_remove "/macOS Install Data"; then
-                    log_success "macOS Install Data ($size_human)"
+                    log_success "macOS Install Data, $size_human"
                 fi
             fi
         else
-            debug_log "Keeping macOS Install Data (only ${age_days} days old, needs 30+)"
+            debug_log "Keeping macOS Install Data, only ${age_days} days old, needs 14+"
         fi
     fi
+    # Clean macOS installer apps (e.g., "Install macOS Sequoia.app")
+    # Only remove installers older than 14 days and not currently running
+    local installer_cleaned=0
+    for installer_app in /Applications/Install\ macOS*.app; do
+        [[ -d "$installer_app" ]] || continue
+        local app_name=$(basename "$installer_app")
+        # Skip if installer is currently running
+        if pgrep -f "$installer_app" > /dev/null 2>&1; then
+            debug_log "Skipping $app_name: currently running"
+            continue
+        fi
+        # Check age (same 14-day threshold as /macOS Install Data)
+        local mtime=$(get_file_mtime "$installer_app")
+        local age_days=$((($(get_epoch_seconds) - mtime) / 86400))
+        if [[ $age_days -lt 14 ]]; then
+            debug_log "Keeping $app_name: only ${age_days} days old, needs 14+"
+            continue
+        fi
+        local size_kb=$(get_path_size_kb "$installer_app")
+        if [[ -n "$size_kb" && "$size_kb" -gt 0 ]]; then
+            local size_human=$(bytes_to_human "$((size_kb * 1024))")
+            debug_log "Cleaning macOS installer: $app_name, $size_human, ${age_days} days old"
+            if safe_sudo_remove "$installer_app"; then
+                log_success "$app_name, $size_human"
+                ((installer_cleaned++))
+            fi
+        fi
+    done
+    stop_section_spinner
+    [[ $installer_cleaned -gt 0 ]] && debug_log "Cleaned $installer_cleaned macOS installer(s)"
     start_section_spinner "Scanning system caches..."
     local code_sign_cleaned=0
     local found_count=0
@@ -72,27 +132,62 @@ clean_deep_system() {
             local current_time
             current_time=$(get_epoch_seconds)
             if [[ $((current_time - last_update_time)) -ge $update_interval ]]; then
-                start_section_spinner "Scanning system caches... ($found_count found)"
+                start_section_spinner "Scanning system caches... $found_count found"
                 last_update_time=$current_time
             fi
         fi
     done < <(run_with_timeout 5 command find /private/var/folders -type d -name "*.code_sign_clone" -path "*/X/*" -print0 2> /dev/null || true)
     stop_section_spinner
-    [[ $code_sign_cleaned -gt 0 ]] && log_success "Browser code signature caches ($code_sign_cleaned items)"
-    safe_sudo_find_delete "/private/var/db/diagnostics/Special" "*" "$MOLE_LOG_AGE_DAYS" "f" || true
-    safe_sudo_find_delete "/private/var/db/diagnostics/Persist" "*" "$MOLE_LOG_AGE_DAYS" "f" || true
+    [[ $code_sign_cleaned -gt 0 ]] && log_success "Browser code signature caches, $code_sign_cleaned items"
+
+    # Optimized: Single pass for diagnostics directory (Special + Persist + tracev3)
+    # Replaces 4 separate find operations with 1 combined operation
+    local diag_base="/private/var/db/diagnostics"
+    if sudo test -d "$diag_base" 2> /dev/null; then
+        while IFS= read -r -d '' file; do
+            if should_protect_path "$file"; then
+                continue
+            fi
+            safe_sudo_remove "$file" || true
+        done < <(sudo find "$diag_base" -maxdepth 5 -type f \( \
+            \( -mtime "+$MOLE_LOG_AGE_DAYS" \) -o \
+            \( -name "*.tracev3" -mtime +30 \) \
+            \) -print0 2> /dev/null || true)
+    fi
     safe_sudo_find_delete "/private/var/db/DiagnosticPipeline" "*" "$MOLE_LOG_AGE_DAYS" "f" || true
     log_success "System diagnostic logs"
     safe_sudo_find_delete "/private/var/db/powerlog" "*" "$MOLE_LOG_AGE_DAYS" "f" || true
     log_success "Power logs"
-    safe_sudo_find_delete "/private/var/db/reportmemoryexception/MemoryLimitViolations" "*" "30" "f" || true
-    log_success "Memory exception reports"
-    start_section_spinner "Cleaning diagnostic trace logs..."
-    local diag_logs_cleaned=0
-    safe_sudo_find_delete "/private/var/db/diagnostics/Persist" "*.tracev3" "30" "f" && diag_logs_cleaned=1 || true
-    safe_sudo_find_delete "/private/var/db/diagnostics/Special" "*.tracev3" "30" "f" && diag_logs_cleaned=1 || true
+    start_section_spinner "Cleaning memory exception reports..."
+    local mem_reports_dir="/private/var/db/reportmemoryexception/MemoryLimitViolations"
+    if sudo test -d "$mem_reports_dir" 2> /dev/null; then
+        # Count and size old files before deletion
+        local file_count=0
+        local total_size_kb=0
+        while IFS= read -r -d '' file; do
+            ((file_count++))
+            local file_size
+            file_size=$(sudo stat -f%z "$file" 2> /dev/null || echo "0")
+            ((total_size_kb += file_size / 1024))
+        done < <(sudo find "$mem_reports_dir" -type f -mtime +30 -print0 2> /dev/null || true)
+
+        # For directories with many files, use find -delete for performance
+        if [[ "$file_count" -gt 0 ]]; then
+            if [[ "${MOLE_DRY_RUN:-0}" != "1" ]]; then
+                sudo find "$mem_reports_dir" -type f -mtime +30 -delete 2> /dev/null || true
+                # Log summary to operations.log
+                if oplog_enabled && [[ "$total_size_kb" -gt 0 ]]; then
+                    local size_human
+                    size_human=$(bytes_to_human "$((total_size_kb * 1024))")
+                    log_operation "clean" "REMOVED" "$mem_reports_dir" "$file_count files, $size_human"
+                fi
+            else
+                log_info "[DRY-RUN] Would remove $file_count old memory exception reports ($total_size_kb KB)"
+            fi
+        fi
+    fi
     stop_section_spinner
-    [[ $diag_logs_cleaned -eq 1 ]] && log_success "System diagnostic trace logs"
+    log_success "Memory exception reports"
 }
 # Incomplete Time Machine backups.
 clean_time_machine_failed_backups() {
@@ -119,7 +214,7 @@ clean_time_machine_failed_backups() {
         echo -e "  ${GREEN}${ICON_SUCCESS}${NC} No incomplete backups found"
         return 0
     fi
-    if tmutil status 2> /dev/null | grep -q "Running = 1"; then
+    if tm_is_running; then
         if [[ "$spinner_active" == "true" ]]; then
             stop_section_spinner
         fi
@@ -176,7 +271,7 @@ clean_time_machine_failed_backups() {
                 local backup_name=$(basename "$inprogress_file")
                 local size_human=$(bytes_to_human "$((size_kb * 1024))")
                 if [[ "$DRY_RUN" == "true" ]]; then
-                    echo -e "  ${YELLOW}${ICON_DRY_RUN}${NC} Incomplete backup: $backup_name ${YELLOW}($size_human dry)${NC}"
+                    echo -e "  ${YELLOW}${ICON_DRY_RUN}${NC} Incomplete backup: $backup_name${NC}, ${YELLOW}$size_human dry${NC}"
                     ((tm_cleaned++))
                     note_activity
                     continue
@@ -186,7 +281,7 @@ clean_time_machine_failed_backups() {
                     continue
                 fi
                 if tmutil delete "$inprogress_file" 2> /dev/null; then
-                    echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Incomplete backup: $backup_name ${GREEN}($size_human)${NC}"
+                    echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Incomplete backup: $backup_name${NC}, ${GREEN}$size_human${NC}"
                     ((tm_cleaned++))
                     ((files_cleaned++))
                     ((total_size_cleaned += size_kb))
@@ -222,7 +317,7 @@ clean_time_machine_failed_backups() {
                     local backup_name=$(basename "$inprogress_file")
                     local size_human=$(bytes_to_human "$((size_kb * 1024))")
                     if [[ "$DRY_RUN" == "true" ]]; then
-                        echo -e "  ${YELLOW}${ICON_DRY_RUN}${NC} Incomplete APFS backup in $bundle_name: $backup_name ${YELLOW}($size_human dry)${NC}"
+                        echo -e "  ${YELLOW}${ICON_DRY_RUN}${NC} Incomplete APFS backup in $bundle_name: $backup_name${NC}, ${YELLOW}$size_human dry${NC}"
                         ((tm_cleaned++))
                         note_activity
                         continue
@@ -231,7 +326,7 @@ clean_time_machine_failed_backups() {
                         continue
                     fi
                     if tmutil delete "$inprogress_file" 2> /dev/null; then
-                        echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Incomplete APFS backup in $bundle_name: $backup_name ${GREEN}($size_human)${NC}"
+                        echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Incomplete APFS backup in $bundle_name: $backup_name${NC}, ${GREEN}$size_human${NC}"
                         ((tm_cleaned++))
                         ((files_cleaned++))
                         ((total_size_cleaned += size_kb))
@@ -251,89 +346,54 @@ clean_time_machine_failed_backups() {
         echo -e "  ${GREEN}${ICON_SUCCESS}${NC} No incomplete backups found"
     fi
 }
-# Local APFS snapshots (keep the most recent).
+# Returns 0 if a backup is actively running.
+# Returns 1 if not running.
+# Returns 2 if status cannot be determined
+tm_is_running() {
+    local st
+    st="$(tmutil status 2> /dev/null)" || return 2
+
+    # If we can't find a Running field at all, treat as unknown.
+    if ! grep -qE '(^|[[:space:]])("Running"|Running)[[:space:]]*=' <<< "$st"; then
+        return 2
+    fi
+
+    # Match: Running = 1;   OR   "Running" = 1   (with or without trailing ;)
+    grep -qE '(^|[[:space:]])("Running"|Running)[[:space:]]*=[[:space:]]*1([[:space:]]*;|$)' <<< "$st"
+}
+
+# Local APFS snapshots (report only).
 clean_local_snapshots() {
     if ! command -v tmutil > /dev/null 2>&1; then
         return 0
     fi
+
+    start_section_spinner "Checking Time Machine status..."
+    local rc_running=0
+    tm_is_running || rc_running=$?
+
+    if [[ $rc_running -eq 2 ]]; then
+        stop_section_spinner
+        echo -e "  ${YELLOW}!${NC} Could not determine Time Machine status; skipping snapshot check"
+        return 0
+    fi
+
+    if [[ $rc_running -eq 0 ]]; then
+        stop_section_spinner
+        echo -e "  ${YELLOW}!${NC} Time Machine is active; skipping snapshot check"
+        return 0
+    fi
+
     start_section_spinner "Checking local snapshots..."
     local snapshot_list
-    snapshot_list=$(tmutil listlocalsnapshots / 2> /dev/null)
+    snapshot_list=$(run_with_timeout 3 tmutil listlocalsnapshots / 2> /dev/null || true)
     stop_section_spinner
     [[ -z "$snapshot_list" ]] && return 0
-    local cleaned_count=0
-    local total_cleaned_size=0 # Estimation not possible without thin
-    local newest_ts=0
-    local newest_name=""
-    local -a snapshots=()
-    while IFS= read -r line; do
-        if [[ "$line" =~ com\.apple\.TimeMachine\.([0-9]{4})-([0-9]{2})-([0-9]{2})-([0-9]{6}) ]]; then
-            local snap_name="${BASH_REMATCH[0]}"
-            snapshots+=("$snap_name")
-            local date_str="${BASH_REMATCH[1]}-${BASH_REMATCH[2]}-${BASH_REMATCH[3]} ${BASH_REMATCH[4]:0:2}:${BASH_REMATCH[4]:2:2}:${BASH_REMATCH[4]:4:2}"
-            local snap_ts=$(date -j -f "%Y-%m-%d %H:%M:%S" "$date_str" "+%s" 2> /dev/null || echo "0")
-            [[ "$snap_ts" == "0" ]] && continue
-            if [[ "$snap_ts" -gt "$newest_ts" ]]; then
-                newest_ts="$snap_ts"
-                newest_name="$snap_name"
-            fi
-        fi
-    done <<< "$snapshot_list"
 
-    [[ ${#snapshots[@]} -eq 0 ]] && return 0
-    [[ -z "$newest_name" ]] && return 0
-
-    local deletable_count=$((${#snapshots[@]} - 1))
-    [[ $deletable_count -le 0 ]] && return 0
-
-    if [[ "$DRY_RUN" != "true" ]]; then
-        if [[ ! -t 0 ]]; then
-            echo -e "  ${YELLOW}!${NC} ${#snapshots[@]} local snapshot(s) found, skipping non-interactive mode"
-            echo -e "  ${YELLOW}${ICON_WARNING}${NC} ${GRAY}Tip: Snapshots may cause Disk Utility to show different 'Available' values${NC}"
-            return 0
-        fi
-        echo -e "  ${YELLOW}!${NC} Time Machine local snapshots found"
-        echo -e "  ${GRAY}macOS can recreate them if needed.${NC}"
-        echo -e "  ${GRAY}The most recent snapshot will be kept.${NC}"
-        echo -ne "  ${PURPLE}${ICON_ARROW}${NC} Remove all local snapshots except the most recent one? ${GREEN}Enter${NC} continue, ${GRAY}Space${NC} skip: "
-        local choice
-        if type read_key > /dev/null 2>&1; then
-            choice=$(read_key)
-        else
-            IFS= read -r -s -n 1 choice || choice=""
-            if [[ -z "$choice" || "$choice" == $'\n' || "$choice" == $'\r' ]]; then
-                choice="ENTER"
-            fi
-        fi
-        if [[ "$choice" == "ENTER" ]]; then
-            printf "\r\033[K" # Clear the prompt line
-        else
-            echo -e " ${GRAY}Skipped${NC}"
-            return 0
-        fi
-    fi
-
-    local snap_name
-    for snap_name in "${snapshots[@]}"; do
-        if [[ "$snap_name" =~ com\.apple\.TimeMachine\.([0-9]{4})-([0-9]{2})-([0-9]{2})-([0-9]{6}) ]]; then
-            if [[ "${BASH_REMATCH[0]}" != "$newest_name" ]]; then
-                if [[ "$DRY_RUN" == "true" ]]; then
-                    echo -e "  ${YELLOW}${ICON_DRY_RUN}${NC} Local snapshot: $snap_name ${YELLOW}dry-run${NC}"
-                    ((cleaned_count++))
-                    note_activity
-                else
-                    if sudo tmutil deletelocalsnapshots "${BASH_REMATCH[1]}-${BASH_REMATCH[2]}-${BASH_REMATCH[3]}-${BASH_REMATCH[4]}" > /dev/null 2>&1; then
-                        echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Removed snapshot: $snap_name"
-                        ((cleaned_count++))
-                        note_activity
-                    else
-                        echo -e "  ${YELLOW}!${NC} Failed to remove: $snap_name"
-                    fi
-                fi
-            fi
-        fi
-    done
-    if [[ $cleaned_count -gt 0 && "$DRY_RUN" != "true" ]]; then
-        log_success "Cleaned $cleaned_count local snapshots, kept latest"
+    local snapshot_count
+    snapshot_count=$(echo "$snapshot_list" | { grep -Eo 'com\.apple\.TimeMachine\.[0-9]{4}-[0-9]{2}-[0-9]{2}-[0-9]{6}' || true; } | wc -l | awk '{print $1}')
+    if [[ "$snapshot_count" =~ ^[0-9]+$ && "$snapshot_count" -gt 0 ]]; then
+        echo -e "  ${YELLOW}${ICON_WARNING}${NC} Time Machine local snapshots: ${GREEN}${snapshot_count}${NC}${GRAY}, Review: tmutil listlocalsnapshots /${NC}"
+        note_activity
     fi
 }
